/*  $Id: owl2dig.pl, v 0.7.0 2005/5/20  zhisheng Exp $


    Author:        Zhisheng Huang
    E-mail:        huang@cs.vu.nl
    WWW:           http://www.cs.vu.nl/~huang
    Copyright (C): 2004-2005 Vrije University Amsterdam

This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    As a special exception, if you link this library with other files,
    compiled with a Free Software compiler, to produce an executable, this
    library does not by itself cause the resulting executable to be covered
    by the GNU General Public License. This exception does not however
    invalidate any other reasons why the executable file might be covered by
    the GNU General Public License.

*/

:- module(owl2dig,
	  [ owl2dig/2,				% +OWL, -DIG  with type: file(FileName), text(Text), elements(ElementList),  with a 'tell' header		
	    owl_dig/2,				% +OWLElements, -DIGElements,  without a header 
          owl_dig_element/2,			% +OWLElement, -DIGElements
          owl2dig_namespace_processing/2,	% +OWLElementsWithHeader, -OWLElementsWithoutHeader
          namespace_processing/1		% +NamespaceList
	  ]).


:-use_module(library('sgml')).
:-use_module(library('dig/dig_process')).


%basic owl2dig format

owl2dig(elements(L), elements(L2)):-
     !,
     owl2dig_namespace_processing(L, L1),
     owl_dig(L1, L3),
     L2 = [element(tells, [xmlns='http://dl.kr.org/dig/lang', 'xmlns:xsi'='http://www.w3.org/2001/XMLSchema-instance', 'xsi:schemaLocation'='http://wasp.cs.vu.nl/sekt/digtest/dig-1.1.xsd'], L3)], 
     true.


owl2dig(elements(L), text(Text)):-
     !,
      owl2dig(elements(L), elements(L2)),
	sformat(Text1, '<?xml version="1.0" encoding="ISO-8859-1"?>', []),
     	elements_xmltext(L2, text(Text3)),  
      sformat(Text4, '<!-- Created with OWL2DIG tool in the XDIG package http://wasp.cs.vu.nl/sekt/dig -->', [ ]),
      concat_atom([Text1, Text3, Text4], Text),
	true.


owl2dig(file(InFile), file(OutFile)):-
      !,
	load_xml_file(InFile, L), 
	owl2dig(elements(L), text(Text)),
      dig_file_out(OutFile, Text).

owl2dig(file(InFile), text(Text)):-
     !,
     load_xml_file(InFile, L), 
     owl2dig(elements(L), text(Text)).


owl2dig(file(InFile), elements(L1)):-
     !,
     load_xml_file(InFile, L), 
     owl2dig(elements(L), elements(L1)).

owl2dig(text(Text1), file(OutFile)):-
     !,
     xmltext_elements(Text1, L),    
     owl2dig(elements(L), text(Text2)),
     dig_file_out(OutFile, Text2),
     true.



owl2dig(text(Text1), text(Text2)):-
     !,
     xmltext_elements(Text1, L),
     owl2dig(elements(L), text(Text2)).

owl2dig(text(Text1), elements(L2)):-
     !,
     xml_elements(Text1, L, _),
     owl2dig(elements(L), elements(L2)).


owl2dig(elements(L), file(OutFile)):-
     !,
     owl2dig(elements(L), text(Text)),
     dig_file_out(OutFile, Text).




owl2dig(X, Y):-
    format(user_error, 'unknown owl2dig translation type: ~w, ~w~n', [X, Y]),
     true.




owl2dig_namespace_processing(L1, L):-
      dig_get_element(L1, E, _L2), 
	E = element(_X, Y, L),
      retractall(namespace(_, _)), % remove all of the old namespace
      namespace_processing(Y),
      !.

owl2dig_namespace_processing(_, _):-!.


dig_file_out(OutFile, Text):-
	open(OutFile, write, Out, []),
      format(Out, '~w', [Text]),
	close(Out),
	true.
    
       


owl_dig([], []):-
                !.

owl_dig([H|T], L):-
                H = element(_,_,_),                
                !,
		    owl_dig_element(H, L1),
		    owl_dig(T, L2),
                append(L1, L2, L).

owl_dig([_H|T], L):-               
                !,
		    owl_dig(T, L).

% ignoring ontology name
owl_dig_element(H, L):-
  %    format(user_error, 'H:~q~n', [H]),
	H = element('owl:Ontology', ['rdf:about' = _C], _),
	!,
      L = [],
	true.


owl_dig_element(H, L):-
	H = element('owl:Class', ['rdf:about' = 'http://www.w3.org/2002/07/owl#Thing'], []),
	!,
      L = [element(top, [],[])],
	true.


owl_dig_element(H, L):-
	H = element('owl:Class', ['rdf:about' = C], []),
	!,
      get_name(C, N),
      L = [element(catom, [name=N], [])],
	true.

owl_dig_element(H, L):-
	H = element('owl:Class', ['rdf:ID' = C], []),
	!,
      get_name(C, N),
      L = [element(catom, [name=N],[])].


owl_dig_element(H, L):-
	H = element('owl:Class', ['rdf:resource' = C], []),
	!,
      get_name(C, N),
      L = [element(catom, [name=N],[])].



owl_dig_element(H, L1):-
	H = element('owl:Class', ['rdf:about' = C], L),
	!,
      E = element('owl:Class', ['rdf:about' = C], []),
      owl_dig(E, L, L1),
	true.

owl_dig_element(H, L1):-
	H = element('owl:Class', ['rdf:ID' = C], L),
	!,
      get_name(C, C1),
      E1 = element(defconcept, [name=C1], []),
	E = element('owl:Class', ['rdf:about' = C], []),
      owl_dig(E, L, L2),
      L1 = [E1|L2].


owl_dig_element(H, L1):-
	H = element('owl:Class', ['rdf:resource' = C], L),
	!,
	E = element('owl:Class', ['rdf:about' = C], []),
      owl_dig(E, L, L1),
	true.

owl_dig_element(H, L2):-
	H = element('owl:Class', [], L),
      dig_get_element(L, E, L1),
      dig_get_element(L1, E1, _),
      E1 = nil,
      !,
      owl_dig_element(E, L2),
	true.


owl_dig_element(H, L2):-
	H = element('owl:Class', [], L),
      dig_get_element(L, E, L1),
      !,
      owl_dig(E, L1, L2),
	true.

owl_dig_element(H, L2):-
	H = element('owl:onProperty', ['rdf:resource' = R], _L),
	!,
      get_name(R, R1),
      L2 =[element(ratom, [name=R1], [])],
	true.


owl_dig_element(E, L2):-
      E = element(Owl, ['rdf:parseType'='Collection'], L),
      owl_dig_voc(Owl, Dig, 2),
	!,
      owl_dig(L, L1),
      L2 = [element(Dig, [], L1)].


owl_dig_element(E, L):-
      E = element(Owl, ['rdf:resource'=C], []),
      owl_dig_voc(Owl, Dig, 2),
	!,
      get_name(C, N),
      L = [element(Dig,[], [element(catom, [name=N], [])])].


owl_dig_element(E, L2):-
      E = element(Owl, [], L),
      owl_dig_voc(Owl, Dig, 2),
	!,
      owl_dig(L, L1),
	L2 = [element(Dig, [], L1)].

owl_dig_element(E, L2):-
      E = element(Owl, ['rdf:parseType'='Collection'], L),
      owl_dig_voc(Owl, Dig, 1),
	!,
      owl_dig(L, L1),
      L2 = [element(Dig, [], L1)].


owl_dig_element(E, L2):-
      E = element(Owl, ['rdf:resource'=C], []),
      owl_dig_voc(Owl, Dig, 1),
	!,
      get_name(C, N),
      L2 =[element(Dig, [], [element(catom, [name=N], [])])].


owl_dig_element(E, L2):-
      E = element(Owl, [], L),
      owl_dig_voc(Owl, Dig, 1),
	!,
      owl_dig(L, L1),
      L2 = [element(Dig, [], L1)].



owl_dig_element(H, L2):-
      H= element('owl:onProperty', [], L),
      !,
      owl_dig(L, L2).


owl_dig_element(H, L2):-
	H = element(Owl, ['rdf:about' = R], []),
      owl_dig_voc(Owl, Dig, 3),
	!,
      get_name(R, R1),
      L2 = [element(Dig, [name=R1],[])],
	true.



owl_dig_element(H, L2):-
	H = element(Owl, ['rdf:ID' = R], []),
	owl_dig_voc(Owl, Dig, 3),
	!,
      get_name(R, R1),
      L2 = [element(Dig, [name=R1], [])],
	true.

owl_dig_element(H, L2):-
	H = element(Owl, ['rdf:ID' = R], L),
      owl_dig_voc(Owl, _Dig, 3),     
	!,
      E=element(Owl, ['rdf:about'= R], []),
      owl_dig(E,L, L2),
	true.


owl_dig_element(H, L2):-
	H = element(Owl, ['rdf:about' = R], L),
      owl_dig_voc(Owl, _Dig, 3),
	!,
      E=element(Owl, ['rdf:about'= R], []),
      owl_dig(E, L, L2),
	true.




owl_dig_element(H, L2):-
	H = element('owl:SymmetricProperty', ['rdf:ID' = R], L),
	!,
      E = element('owl:ObjectProperty', ['rdf:about' = R], []),
      get_name(R, R1),
      owl_dig(E, L, L1),
      E2 = element(equalr, [], [element(ratom, [name=R1],[element(inverse, [], [element(ratom, [name=R1],[])])])]),
      L2 =[E2|L1],
	true.


owl_dig_element(H, L2):-
	H = element('owl:TransitiveProperty', ['rdf:ID' = R], L),
	!,
      E = element('owl:ObjectProperty', ['rdf:about' = R], []),
      get_name(R, R1),
      E1 = element(transitive, [], [element(ratom, [name=R1], [])]),
      owl_dig(E,L, L1),
	L2 = [E1|L1].


owl_dig_element(H, L2):-
	H = element('owl:InverseFunctionalProperty', ['rdf:ID' = R], L),
	!,
      E = element('owl:ObjectProperty', ['rdf:about' = R], []),
      get_name(R, R1),
      E1 = element(equalr, [], [element(ratom, [name=R1], []), element(inverse,[], [element(ratom, [name=R1],[])])]),
      owl_dig(E, L, L1),
	L2 = [E1|L1].


owl_dig_element(H, L2):-
	H = element('owl:Restriction', [], L),
	!,
      E = element('owl:onProperty', _, _),
      select(E, L, L1),
      owl_dig(E, L1, L2),
	true.

owl_dig_element(H, L2):-
	H = element('owl:Thing', ['rdf:about' = I], []),
      !,
      get_name(I, I1),
      L2 =[element(individual, [name=I1], [])].



owl_dig_element(H, L2):-
	H = element('owl:Thing', ['rdf:about' = I], L),
      !,
      get_name(I, I1),
      E1 = element(instanceof, [], [element(individual, [name=I1], []), element(top, [],[])]),
      E= element('owl:Thing', ['rdf:about'=I], []),
      owl_dig(E, L, L1),
	L2 = [E1|L1].

owl_dig_element(H, L2):-
	H = element('rdf:Description', ['rdf:about' = I], L),
      !,
      get_name(I, I1),
	E1 = element('owl:Thing', ['rdf:about' = I1], []),
      owl_dig(E1, L, L2).



owl_dig_element(H, L2):-
	(H = element('rdfs:Class', ['rdf:ID'=I], []);H = element('rdfs:Class', ['rdf:resource'=I], [])),
	!,
      get_name(I, I1),
      L2 =[element(catom, [name=I1], [])],
	true.




owl_dig_element(H, L2):-
	(H = element(C, ['rdf:ID'=I], []);H = element(C, ['rdf:resource'=I], [])),
	!,
     get_name(I, I1),
     get_name(C, C1),
     L2 = [element(instanceof, [], [element(individual, [name=I1], []), element(catom, [name=C1], [])])],
	true.





owl_dig_element(H, L2):-
	H = element(C, ['rdf:datatype'=_], [N]),
	!,
     get_name(C, C1),
     L2 = [element(attribute, [name=C1],[]), element(sval, [], N)].


owl_dig_element(H, L2):-
	H = element('rdfs:Class', ['rdf:ID'=I], L),
	!,
     get_name(I, I1),
      E1 = element(defconcept, [name=I1], []),
      E = element('owl:Class', ['rdf:resource'=I],[]),
      owl_dig(E, L, L1),
	L2 = [E1|L1],
	true.


owl_dig_element(H, L2):-
	H = element('rdf:Description', ['rdf:ID'=I], L),
	!,
     get_name(I, I1),
      E1 = element(defconcept, [name=I1],[]),
      E = element('owl:Class', ['rdf:resource'=I], []),
      owl_dig(E, L, L1),
	L2 = [E1|L1],
	true.




owl_dig_element(H, L2):-
	H = element(C, ['rdf:ID'=I], L),
	!,
     get_name(I, I1),
     get_name(C, C1),
      E1 = element(instanceof, [], [element(individial, [name=I1], []), element(catom, [name=C1], [])]),
      E = element(C, ['rdf:resource'=I], []),
      owl_dig(E, L, L1),
	L2 =[E1|L1],
	true.


owl_dig_element(H, L3):-
	H = element('owl:AllDifferent', _A, L),
      dig_get_element(L, E, _L1),
      E = element('owl:distinctMembers', ['rdf:parseType'='Collection'], L2),
	!,
      owl_dig_processing(alldifferent, L2, L3),
	true.

owl_dig_element(H, L):-
      H = element(_, _, _),
	!, 
	format(user_error, '***untranslated element: ~q~n', [H]),
      L = [],
      true.


owl_dig_element(_, []):-!.




owl_dig(_C, [], []):-!.

owl_dig(E1, L, L3):-
      dig_get_element(L, E, L1),
      E = element(Owl, ['rdf:resource'=C2], []),
      owl_dig_voc(Owl, Dig, 1),
      E1 = element('owl:Class', ['rdf:about' = C],[]),
	!,
     get_name(C2, C3),
     get_name(C, C1),
     E2 = element(equalc, [],[element(catom, [name=C1], []), element(Dig, [], [element(catom, [name=C3],[])])]),
      owl_dig(E1, L1, L2),
      L3 = [E2|L2].


 
owl_dig(E1, L, L3):-
      dig_get_element(L, E, L1),
      (E = element(Owl, ['rdf:resource'=C2], []); E = element(Owl, ['rdf:about'=C2], [])),
      owl_dig_voc(Owl, Dig, 2),
	!,
      owl_dig_element(E1, L2),
      get_name(C2, C1),
      append(L2, [element(catom, [name=C1], [])], L5),
      E3 = element(Dig, [], L5),
      owl_dig(E1, L1, L4),
      L3 = [E3|L4].


owl_dig(E1, L, L5):-
      dig_get_element(L, E, L1),
      E = element(Owl, _X, L2),
      owl_dig_voc(Owl, Dig, 1),
      E1 = element('owl:Class', ['rdf:about' = C],[]),
	!,
      get_name(C, C1),
	owl_dig(L2, L3),
      E4 = element(equalc, [], [element(catom, [name=C1],[]), element(Dig, [], L3)]),
      owl_dig(E1, L1, L4),
	L5 = [E4|L4].


owl_dig(E1, L, L5):-
      dig_get_element(L, E, L1),
      E = element(Owl, _X, L2),
      owl_dig_voc(Owl, Dig, 1),
	!,
	owl_dig(L2, L3),
      E4 = element(Dig, [], L3),
      owl_dig(E1, L1, L4),
	L5 = [E4|L4].




owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element(Owl, _X, L2),
      owl_dig_voc(Owl, Dig, 2),
	!,
      owl_dig_element(E1, L3),
	owl_dig(L2, L4),
      append(L3, L4, L5),
      E4 = element(Dig, [], L5),
      owl_dig(E1, L1, L6),
	L7 = [E4|L6].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:cardinality', _, [N]),
	!,
      owl_dig_element(E1, L2),
      append(L2, [element(top, [],[])], L4),
      E6 = element(and, [], [element(atleast, [num=N],L4), element(atmost, [num=N], L4)]),
      owl_dig(E1, L1, L6),
	L7 = [E6|L6].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:minCardinality', _, [N]),
	!,
      owl_dig_element(E1, L2),
	append(L2, [element(top, [],[])], L4),
      E6 = element(atleast, [num=N], L4),
      owl_dig(E1, L1, L6),
	L7 = [E6|L6].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:maxCardinality', _, [N]),
	!,
      owl_dig_element(E1, L2),
	append(L2, [element(top, [],[])], L4),
      E6 = element(atleast, [num=N], L4),
      owl_dig(E1, L1, L6),
	L7 = [E6|L6].




owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:hasValue',  ['rdf:resource'=V], []),
	!,
      owl_dig_element(E1, L2),
      get_name(V, V1),
      E2 =element(iset, [], [element(individual, [name=V1], [])]),
      owl_dig(E1, L1, L3),
	append(L2, [E2], L4),
      append(L4, L3, L7).

owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:inverseOf',  ['rdf:resource'=V], []),
	!,
      get_name(V, V1),
      owl_dig_element(E, L2),
      E2 = element(equalr, [], [element(ratom, [name=V1],[]), element(inverse, [], L2)]),      
      owl_dig(E1, L1, L3),
	L7 = [E2|L3].



owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:hasValue', [], L2),
	!,
      owl_dig_element(E1, L3),
      owl_dig(L2, L4),
      E2= element(iset,[], L4),
      owl_dig(E1, L1, L5),
	append(L3, [E2], L6),
      append(L6, L5, L7).



owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = 'http://www.w3.org/2002/07/owl#FunctionalProperty'],[]), 
	!,
      owl_dig_element(E1, L2),
      E4 = element(functional, [], L2),
      owl_dig(E1, L1, L4),
	L7 = [E4|L4].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = 'http://www.w3.org/2002/07/owl#ObjectProperty'],[]), 
	!,
      owl_dig_element(E1, L2),
      E4 = element(functional, [], L2),
      owl_dig(element(E1), L1, L4),
    	L7 = [E4|L4].



owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = 'http://www.w3.org/2002/07/owl#DatatypeProperty'],[]), 
	!,
      owl_dig_element(E1, L2),
      E4 = element(rangeint, [], L2),
      owl_dig(E1, L1, L4),
	L7 = [E4|L4].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = 'http://www.w3.org/2002/07/owl#TransitiveProperty'],[]), 
	!,
      owl_dig_element(E1, L2),
      E4 = element(transitive, [], L2),
      owl_dig(E1, L1, L4),
	L7 = [E4|L4].

owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = 'http://www.w3.org/2002/07/owl#SymmetricProperty'],[]), 
	!,
      owl_dig_element(E1, L2),
      E4 = element(inverse, [], L2),
      owl_dig(E1, L1, L4),
	L7 = [E4|L4].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = 'http://www.w3.org/2002/07/owl#Thing'],[]), 
      E1 = element('owl:Thing', ['rdf:about' = I], []),
	!,
      get_name(I, N),
      E4 = element(defindividual, [name=N], []),
      owl_dig(E1, L1, L4),
	L7 =[E4|L4].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource' = R2],[]), 
      (E1 = element('owl:DatatypeProperty', ['rdf:about' = R1], []);E1 = element('owl:ObjectProperty', ['rdf:about' = R1], [])),
	!,
      get_name(R1, N1),
      get_name(R2, N2),
      E4 = element(impliesr, [], [element(ratom, [name=N1], []), element(ratom, [name=N2], [])]),
      owl_dig(E1, L1, L4),
	L7 =[E4|L4].

owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('owl:inverseOf', [], L2), 
      E1 = element('owl:ObjectProperty', ['rdf:about' = R1], []),
	!,
      get_name(R1, N1),
      dig_get_element(L2, E2,_),
      owl_dig_element(E2, L3),
      E4 = element(equalr, [], [element(ratom, [name=N1], [])|L3]),
      owl_dig(E1, L1, L4),
	L7 =[E4|L4].




owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element(A, ['rdf:datatype' = 'http://www.w3.org/2001/XMLSchema#integer'],[V]), 
      E1 = element('owl:Thing', ['rdf:about' = I], []),
	!,
      get_name(I, N1),
      get_name(A, N2),
      E4 = element(value, [], [element(individual, [name=N1], []), element(attribute, [name=N2], []), element(sval, [], [V])]),
      owl_dig(E1, L1, L4),
	L7 = [E4|L4].

owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E =  element('owl:hasValue', ['rdf:datatype'='http://www.w3.org/2001/XMLSchema#string'], [I]),
      E1 = element('owl:onProperty', [], LE1),
      dig_get_element(LE1, E2, _),
      (E2 = element('owl:DatatypeProperty', ['rdf:ID'=A], []);E2 =  element('owl:DatatypeProperty', ['rdf:about'=A], [])),
	!,
      get_name(I, N1),
      get_name(A, N2),
      E4 = element(value, [], [element(attribute, [name=N1],[]), element(sval, [], [N2])]),
      owl_dig(E1, L1, L4),
	L7 =[E4|L4].





owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E =  element(Owl, ['rdf:resource'=C], []),
      owl_dig_voc(Owl, Dig, 4),
	!,
      owl_dig_element(E1, L2),
      get_name(C, C1),
      append(L2, [element(catom, [name=C1], [])], L3),
      E4 = element(Dig, [], L3),
      owl_dig(E1, L1, L4),
	L7 =[E4|L4].




owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E =  element(Owl, [], L2),
      owl_dig_voc(Owl, Dig, 4),
	!,
      owl_dig_element(E1, L3),
      owl_dig(L2, L4),
      append(L3, L4, L5),
      E4 = element(Dig, [], L5),
      owl_dig(E1, L1, L6),
	L7 = [E4|L6].




owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdfs:subPropertyOf', [], L2),
 	!,
      owl_dig_element(E1, L3),
      owl_dig(L2, L4),
      append(L3, L4, L5),
      E4 = element(impliesr, [], L5),
      owl_dig(E1, L1, L6),
	L7 = [E4|L6].


owl_dig(E1, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdfs:subPropertyOf', ['rdf:resource' = R], []),
 	!,
      owl_dig_element(E1, L2),
      get_name(R, R1),
      append(L2, [element(ratom, [name=R1], [])], L3),
      E4 = element(impliesr,[], L3),
	owl_dig(E1, L1, L4),
	L7 = [E4|L4].


% ignoring the label and comment
owl_dig(C, L, L7):-
      dig_get_element(L, E, L1),
      (E = element('rdfs:label', _, [_Label]);E = element('rdfs:comment', _, [_Comment])),
	!,
	owl_dig(C, L1, L7),
      true.



owl_dig(C, L, L7):-
      dig_get_element(L, E, L1),
      E = element('rdf:type', [], L2),
	!,
      owl_dig_element(C, L3),
      owl_dig(L2, L4),
      append(L3, L4, L5),
      E6 = element(instanceof, [], L5),
      owl_dig(C, L1, L6),
      L7 = [E6|L6],
	true.


owl_dig(C, L, L7):-
      (C = element('owl:Class', ['rdf:resource' = I], []);C = element('rdf:Description', ['rdf:resource' = I], [])),
      dig_get_element(L, E, L1),
      E = element('rdf:type', ['rdf:resource'= I2], []),
	!,
     get_name(I, N1),
     get_name(I2,N2),
       E4 = element(impliesc, [], [element(catom, [name=N1], []),element(catom, [name=N2], [])]),
      owl_dig(C, L1, L4),
      L7 =[E4|L4],
	true.

owl_dig(C, L, L7):-
      C = element('rdf:Description', ['rdf:resource' = I], []),
      dig_get_element(L, E, L1),
      E = element(R, ['rdf:resource'= I2], []),
	!,
     get_name(I, N1),
     get_name(I2,N2),
     get_name(R, N3),
      E4 = element(related, [], [element(individual, [name=N1], []),element(ratom, [name=N3], []), element(individual,[name=N2], [])]),
      owl_dig(C, L1, L4),
      L7 =[E4|L4],
	true.


owl_dig(C, L, L7):-
      C = element(_C1, ['rdf:resource' = I], []),
      dig_get_element(L, E, L1),
      E = element(_C2, ['rdf:datatype'= _], _),
	!,
      get_name(I, I1),
      owl_dig_element(E, L2),
      E4 = element(value, [], [element(individual, [name=I1],[])|L2]),
      owl_dig(C, L1, L4),
      L7=[E4|L4],
	true.
 

owl_dig(_C, L, L7):-
     dig_get_element(L, E, _L1),
     E = nil,
     !,
     L7 =[].

   

owl_dig(C, L, L7):-
	!, 
	format(user_error, '***untranslated list: ~q:~q~n', [C, L]),
      L7 = [],
      true.


owl_dig_voc('owl:intersectionOf', and, 1):-!.
owl_dig_voc('owl:complementOf', not, 1):-!.
owl_dig_voc('owl:unionOf', or, 1):-!.
owl_dig_voc('owl:oneOf', iset, 1):-!.


owl_dig_voc('rdfs:subClassOf', impliesc, 2):-!.
owl_dig_voc('owl:disjointWith', disjoint, 2):-!.
owl_dig_voc('owl:equivalentClass', equalc, 2):-!.
owl_dig_voc('owl:allValuesFrom', all, 2):-!.
owl_dig_voc('owl:someValuesFrom', some, 2):-!.

owl_dig_voc('owl:ObjectProperty', ratom, 3):-!.
owl_dig_voc('owl:FunctionalProperty', ratom, 3):-!.
owl_dig_voc('owl:DatatypeProperty', attribute, 3):-!.
owl_dig_voc('rdfs:range', range, 4):-!.
owl_dig_voc('rdfs:domain', domain, 4):-!.


owl_dig_processing(alldifferent, [], L7):-
       !,
       L7 = [].

owl_dig_processing(alldifferent, L, L7):-
       dig_get_element(L, E, L1),
       E = element(C,['rdf:about'=I],[]),
       !,
       get_name(I, I1),
     	 get_name(C, C1),
       L2 = [element(defindividual, [name=I1], []), element(instanceof, [], [element(individual, [name=I1], []), element(catom, [name=C1], [])])],
       owl_dig_processing(alldifferent, L1, L3),
	 append(L2, L3, L7),
	 true.

owl_dig_processing(alldifferent, _, L7):-
       L7=[].



namespace_processing([]):-!.
namespace_processing([N = S|T]):-
      assert(namespace(N, S)),
      !,
	namespace_processing(T).

namespace_processing([_|T]):-
      namespace_processing(T).


get_namespace(xmlns, M):-
     namespace(xmlns, M),
     nonvar(M),
     !.

get_namespace(N, M):-
     sub_string(N, X, _Y, _Z, ':'),
     X > 0,
     namespace(N, M),
     nonvar(M),
     !.


get_namespace(S, N):-
     concat_atom(['xmlns:', S], NS),
     namespace(NS, N),
     nonvar(N),
     !.

get_namespace(_, ''):-!.

% type1: #Name
get_name(N, N1):-
         sub_string(N, X, _Y, Z, '#'),
	   X = 0,
         !,
         sub_string(N, 1, Z, _Z1, N2),
         get_namespace('xml:base', NS),
         concat_atom([NS, N2], N1).

% type2: BASE#Name
get_name(N, N1):-
         sub_string(N, X, _Y, _Z, '#'),
	   X > 0,
         !,
         N1= N.


% type3: NS:Name
get_name(N, N1):-
         sub_string(N, X, _Y, Z, ':'),
	   X > 0,
         !,
         sub_string(N, 0, X, _Z1, NS),
         X1 is X + 1, 
         sub_string(N, X1, Z, _Z2, M),
         get_namespace(NS, Text1),
         concat_atom([Text1, M], N1).

% type4: Name
get_name(N, N1):-
         !,        
         get_namespace('xml:base', NS),
         concat_atom([NS, N], N1).



    
    

    	
